$ontext
===================================================================================
Name: HRCModel.gms
Function: Mathematical formulation.  Based on the SPD model formulation v4.3
Developed by: Tuong Nguyen  (Electricity Authority, New Zealand)
Last modified: 01 December 2011
===================================================================================
$offtext

*Primary sets
SETS
*Period
TradeDate(*)                     'Dates definition'
TradePeriod(*)                   'Trading periods of a day'
Season(*)                        'Seasonal branch rating'

*Offer
Offer(*)                         'SPD Offer - MarketNode'
Scheme(*)                        'Trade group definition - used in calculation of total energy output GWh'
TradeBlock(*)                    'Number of offer tranches'

*Grid Data Sets
Island(*)                        'Island definition'
Node(*)                          'SPD Node definitions'
Branch(*)                        'Branch ID'
FlowDirection(*)                 'Direction of a flow - Forward or Backward'
LossSegment(*)                   'Number of loss segment'
BranchGroupConstraint(*)              'Transmission constraint definition'
;
Alias (Node,FromNode,ToNode), (Offer,Offer1,Offer2);

*Subsets and multidimentional sets
SETS
TradePeriodSeason(TradeDate,TradePeriod,Season) 'Trade period and season mapping'

*Demand and Offer Sets
SchemeOffer(Scheme,Offer)                 'Mapping scheme and offer'
OfferNode(Offer,Node)                     'Mapping offer and node'

*Grid Data Sets
RefNode(Node)                             'Reference Node for each island'
ACBranch(Branch)                          'AC branches'
DCBranch(Branch)                          'Branches that are HVDC'
NodeIsland(Node,Island)                   'Mapping Node and Island'
BranchNodeFr(Branch,Node)                 'Sending Node Definition'
BranchNodeTo(Branch,Node)                 'Receiving Node Definition'
DCDirection(Branch,FlowDirection)         'Direction defintion for HVDC poles S->N : Forward and N->S : Backward'
;

PARAMETERS
SchemeType(Scheme)                                       'Type of a Scheme: 1 = Profiled output, 2 = Thermal, 3 = TKU and RPO and MAN, 4 = Waikato, 5 = Waitaki and Clutha'

*Static Branch Data
BranchCapacity(Branch,Season)                            'Capacity of a branch'
BranchSusceptance(Branch)                                'Branch Susceptance - B'
BranchResistance(Branch)                                 'Branch Resistance - R'
BranchFixedLoss(Branch)                                  'Branch Fixed Loss'
BranchNumLossTranches(Branch)                            'Number of loss tranches - used to piecewise the loss curve'
BranchOpenStatus(Branch)                                 'Branch open status, 1 = Open'
BranchClosedStatus(Branch)                               'Branch open status, 1 = Open'

ValidLossSegment(Branch,LossSegment)                     'Flag of valid loss segment'

ACBranchLossMW(Branch,LossSegment,Season)                'MW capacity of each loss segment of AC branches'
ACBranchLossFactor(Branch,LossSegment,Season)            'Loss factor of each loss segment of AC branches'

HVDCBreakPointMWLoss(Branch,LossSegment,Season)          'HVDC piece-wise loss segment break point'
HVDCBreakPointMWFlow(Branch,LossSegment,Season)          'HVDC piece-wise flow segment break point'

*Static Branch constraint data
BranchGroupConstraintFactor(BranchGroupConstraint,Branch)          'Branch security constraint factors (sensitivities) for the current trading period'
BranchGroupConstraintSense(BranchGroupConstraint)                  'Branch security constraint sense for the current trading period (-1:<=, 0:= 1:>=)'
BranchGroupConstraintLimit(BranchGroupConstraint)                  'Branch security constraint limit for the current trading period'

*Trading period demand and offer data
TradePeriodNodeDemand(TradeDate,TradePeriod,Node)                    'Demand data'
TradePeriodOfferMW(TradeDate,TradePeriod,Offer,TradeBlock)           'Offer MW data'
TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,TradeBlock)        'Offer Price data'

*Switching Flag
UseBranchFlowMIP(TradeDate,TradePeriod)                  'Check if a trading period need to be solved using MIP branch flow constraint'
;

SCALARS
*Volation penalties
*These violation penalties are not specified in the model formulation document (ver.4.3) but are specified in the
*document "Resolving Infeasibilities & High Spring Washer Price situations - an overview" available at www.systemoperator.co.nz/n2766,264.html

DeficitBusGenerationPenalty                      'Bus deficit violation penalty'                      /500000/
SurplusBusGenerationPenalty                      'Bus surplus violation penalty'                      /500000/
DeficitBranchGroupConstraintPenalty              'Deficit branch group constraint violation penalty'  /500000/
SurplusBranchGroupConstraintPenalty              'Surplus branch group constraint violation penalty'  /500000/
DeficitBranchFlowPenalty                         'Deficit branch flow violation penalty'              /500000/
SurplusBranchFlowPenalty                         'Surplus branch flow violation penalty'              /500000/

*Setting scalars
*Use to swith between different modes
UseHVDCLinkLimits                                 '1 = HVDC Branch Limit Constraints is applied'                                      /1/
UseACBranchLimits                                 '1 = AC Branch Limit Constraints is applied'                                        /1/
ResolveCircularBranchFlows                        '1 = a trading period will be resolved if MIP branch flow constraint is applied'    /1/
ResolveHVDCNonPhysicalLosses                      '1 = a trading period will be resolved if MIP branch loss constraint is applied'    /1/
UseHRCRun                                         '1 = HRC Run applied'                                                               /1/
;

VARIABLES
NETCOST                                                                          'Defined as the total of producer costs adjusted for penalty costs'

*Network
ACNODENETINJECTION(TradeDate,TradePeriod,Node)                                   'MW injection at buses corresponding to AC nodes'
ACBRANCHFLOW(TradeDate,TradePeriod,Branch)                                       'MW flow on undirected AC branch'
ACNODEANGLE(TradeDate,TradePeriod,Node)                                          'Bus voltage angle'

POSITIVE VARIABLES
*Generation
GENERATION(TradeDate,TradePeriod,Offer)                                          'Total MW generation scheduled from an offer'
GENERATIONBLOCK(TradeDate,TradePeriod,Offer,TradeBlock)                          'MW generation scheduled from the individual trade blocks of an offer'
GENERATIONSCHEMERATIO(TradeDate,TradePeriod,Scheme)                              'Ratio of scheme output to scheme offer --> hyrdo plants of the same shceme generate at proportional rate - HRC'

*Network
HVDCLINKFLOW(TradeDate,TradePeriod,Branch)                                               'MW flow at the sending end scheduled for the HVDC link'
HVDCLINKLOSSES(TradeDate,TradePeriod,Branch)                                             'MW losses on the HVDC link'
LAMBDA(TradeDate,TradePeriod,Branch,LossSegment)                                         'Non-negative weight applied to the breakpoint of the HVDC link'
ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,Branch,FlowDirection)                         'MW flow on the directed branch'
ACBRANCHLOSSESDIRECTED(TradeDate,TradePeriod,Branch,FlowDirection)                       'MW losses on the directed branch'
ACBRANCHFLOWBLOCKDIRECTED(TradeDate,TradePeriod,Branch,LossSegment,FlowDirection)        'MW flow on the different blocks of the loss curve'
ACBRANCHLOSSESBLOCKDIRECTED(TradeDate,TradePeriod,Branch,LossSegment,FlowDirection)      'MW losses on the different blocks of the loss curve'

*Violations
TOTALPENALTYCOST                                                                 'Total violation costs'
DEFICITBUSGENERATION(TradeDate,TradePeriod,Node)                                 'Deficit generation at a bus in MW'
SURPLUSBUSGENERATION(TradeDate,TradePeriod,Node)                                 'Surplus generation at a bus in MW'
DEFICITBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint)          'Deficit branch security constraint in MW'
SURPLUSBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint)          'Surplus branch security constraint in MW'
DEFICITBRANCHFLOW(TradeDate,TradePeriod,Branch)                                  'Deficit branch flow in MW'
SURPLUSBRANCHFLOW(TradeDate,TradePeriod,Branch)                                  'Surplus branch flow in MW'
;

SOS1 VARIABLES
ACBRANCHFLOWDIRECTED_INTEGER(TradeDate,TradePeriod,Branch,FlowDirection)       'Integer variables used to select branch flow direction in the event of circular branch flows (3.8.1)'
HVDCLINKFLOWDIRECTION_INTEGER(TradeDate,TradePeriod,FlowDirection)             'Integer variables used to select the HVDC branch flow direction on in the event of S->N (forward) and N->S (reverse) flows (3.8.2)'
;

SOS2 VARIABLES
LAMBDAINTEGER(TradeDate,TradePeriod,Branch,LossSegment)                        'Integer variables used to enforce the piecewise linear loss approxiamtion on the HVDC links'
;

EQUATIONS
ObjectiveFunction                                                              'Objective function of the dispatch model (4.1.1.1)'

*ViolationCost
TotalViolationCostDefinition                                                   'Defined as the sum of the individual violation costs'

*Offer and purchase definitions
GenerationClearedMax(TradeDate,TradePeriod,Offer,TradeBlock)                   'Generation(g,j) <= GenerationOfferMW(g,j) - (3.1.1.1)'
GenerationOfferDefintion(TradeDate,TradePeriod,Offer)                          'Definition of generation provided by an offer (3.1.1.2)'

SchemeRatio(TradeDate,TradePeriod,Scheme)                                      '1st Definition of  scheme generation ratio - (This is for HRC)'
SchemeGenerationRatio(TradeDate,TradePeriod,Scheme,Offer)                      '2nd Definition of  scheme generation ratio - (This is for HRC)'

*Network - HVDC
HVDCLinkMaximumFlow(TradeDate,TradePeriod,Branch)                                    'Maximum flow on each HVDC link (3.2.1.1)'
HVDCLinkLossDefinition(TradeDate,TradePeriod,Branch)                                 'Definition of losses on the HVDC link (3.2.1.2)'
HVDCLinkFlowDefinition(TradeDate,TradePeriod,Branch)                                 'Definition of MW flow on the HVDC link (3.2.1.3)'
LambdaDefinition(TradeDate,TradePeriod,Branch)                                       'Definition of weighting factor (3.2.1.4)'

*Network - HVAC
ACNodeNetInjectionDefinition1(TradeDate,TradePeriod,Node)                           '1st definition of the net injection at buses corresponding to AC nodes (3.3.1.1)'
ACNodeNetInjectionDefinition2(TradeDate,TradePeriod,Node)                           '2nd definition of the net injection at buses corresponding to AC nodes (3.3.1.2)'
ACBranchMaximumFlow(TradeDate,TradePeriod,Branch,FlowDirection)                     'Maximum flow on the AC branch (3.3.1.3)'
ACBranchFlowDefinition(TradeDate,TradePeriod,Branch)                                'Relationship between directed and undirected branch flow variables (3.3.1.4)'
LinearLoadFlow(TradeDate,TradePeriod,Branch)                                        'Equation that describes the linear load flow (3.3.1.5)'
ACBranchBlockLimit(TradeDate,TradePeriod,Branch,LossSegment,FlowDirection)          'Limit on each AC branch flow block (3.3.1.6)'
ACDirectedBranchFlowDefinition(TradeDate,TradePeriod,Branch,FlowDirection)          'Composition of the directed branch flow from the block branch flow (3.3.1.7)'
ACBranchLossCalculation(TradeDate,TradePeriod,Branch,LossSegment,FlowDirection)     'Calculation of the losses in each loss segment (3.3.1.8)'
ACDirectedBranchLossDefinition(TradeDate,TradePeriod,Branch,FlowDirection)          'Composition of the directed branch losses from the block branch losses (3.3.1.9)'
ACReferenceNodeAngle(TradeDate,TradePeriod,Node)                                    'Reference node angles are set equal to zero (3.3.1.10)'

*Branch security constraints
BranchSecurityConstraintLE(TradeDate,TradePeriod,BranchGroupConstraint)                  'Branch security constraint with LE sense (3.5.1.5a)'
BranchSecurityConstraintGE(TradeDate,TradePeriod,BranchGroupConstraint)                  'Branch security constraint with GE sense (3.5.1.5b)'
BranchSecurityConstraintEQ(TradeDate,TradePeriod,BranchGroupConstraint)                  'Branch security constraint with EQ sense (3.5.1.5c)'

*SOS1 and SOS2 constraints
HVDCLinkFlowIntegerDefinition1(TradeDate,TradePeriod)                                'Definition of the integer HVDC link flow variable (3.8.2a)'
HVDCLinkFlowIntegerDefinition2(TradeDate,TradePeriod,FlowDirection)                  'Definition of the integer HVDC link flow variable (3.8.2b)'
LambdaIntegerDefinition1(TradeDate,TradePeriod,Branch)                               'Definition of weighting factor when branch integer constraints are needed (3.8.3a)'
LambdaIntegerDefinition2(TradeDate,TradePeriod,Branch,LossSegment)                   'Definition of weighting factor when branch integer constraints are needed (3.8.3b)'
ACDirectedBranchFlowIntegerDefinition1(TradeDate,TradePeriod,Branch)                'Integer constraint to enforce a flow direction on loss AC branches in the presence of circular branch flows or non-physical losses (3.8.1a)'
ACDirectedBranchFlowIntegerDefinition2(TradeDate,TradePeriod,Branch,FlowDirection)  'Integer constraint to enforce a flow direction on loss AC branches in the presence of circular branch flows or non-physical losses (3.8.1b)'

;


*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------*
*MATHEMATICAL MODEL                                                                                                                                                          *
*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------*

*Objective function of the dispatch model (4.1.1.1)
ObjectiveFunction..
NETCOST =e= 0
+ Sum[(TradeDate,TradePeriod,Offer,TradeBlock), GENERATIONBLOCK(TradeDate,TradePeriod,Offer,TradeBlock) * TradePeriodOfferPrice(TradeDate,TradePeriod,Offer,TradeBlock)]
+ TOTALPENALTYCOST;

*Defined as the sum of the individual violation costs
TotalViolationCostDefinition..
TOTALPENALTYCOST =e= 0
+ Sum[(TradeDate,TradePeriod,Node), DeficitBusGenerationPenalty * DEFICITBUSGENERATION(TradeDate,TradePeriod,Node)]
+ Sum[(TradeDate,TradePeriod,Node), SurplusBusGenerationPenalty * SURPLUSBUSGENERATION(TradeDate,TradePeriod,Node)]
+ Sum[(TradeDate,TradePeriod,Branch), SurplusBranchGroupConstraintPenalty * SURPLUSBRANCHFLOW(TradeDate,TradePeriod,Branch)]
+ Sum[(TradeDate,TradePeriod,BranchGroupConstraint), SurplusBranchGroupConstraintPenalty * SURPLUSBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint)]
+ Sum[(TradeDate,TradePeriod,BranchGroupConstraint), DeficitBranchGroupConstraintPenalty * DEFICITBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint)]
;


*-----------------------------GENERATION EQUATION CONSTRAINTS-----------------------------------------------------------------------------------------------------------------

*Definition of generation provided by an offer (3.1.1.1)
GenerationClearedMax(TradeDate,TradePeriod,Offer,TradeBlock)..
GENERATIONBLOCK(TradeDate,TradePeriod,Offer,TradeBlock) =l= TradePeriodOfferMW(TradeDate,TradePeriod,Offer,TradeBlock);

*Definition of generation provided by an offer (3.1.1.2)
GenerationOfferDefintion(TradeDate,TradePeriod,Offer)..
GENERATION(TradeDate,TradePeriod,Offer) =e= sum[TradeBlock, GENERATIONBLOCK(TradeDate,TradePeriod,Offer,TradeBlock)];

*Definition of Scheme generation ratio 1(HRC)
SchemeRatio(TradeDate,TradePeriod,Scheme) $[(Sum(Offer2 $ SchemeOffer(Scheme,Offer2), 1) > 1) and UseHRCRun]..
Sum[Offer $ SchemeOffer(Scheme,Offer), GENERATION(TradeDate,TradePeriod,Offer)] =e=
GENERATIONSCHEMERATIO(TradeDate,TradePeriod,Scheme) * Sum[(Offer1,TradeBlock) $ SchemeOffer(Scheme,Offer1), TradePeriodOfferMW(TradeDate,TradePeriod,Offer1,TradeBlock)]
;

*Definition of Scheme generation ratio 2(HRC)
SchemeGenerationRatio(TradeDate,TradePeriod,Scheme,Offer) $ [SchemeOffer(Scheme,Offer) and (Sum[Offer1 $ SchemeOffer(Scheme,Offer1), 1] > 1) and UseHRCRun]..
GENERATION(TradeDate,TradePeriod,Offer) =e=
GENERATIONSCHEMERATIO(TradeDate,TradePeriod,Scheme) * Sum[TradeBlock, TradePeriodOfferMW(TradeDate,TradePeriod,Offer,TradeBlock)];

*-----------------------------GENERATION EQUATION CONSTRAINTS END-------------------------------------------------------------------------------------------------------------





*-----------------------------HVDC TRANSMISSION EQUATION CONSTRAINTS-----------------------------------------------------------------------------------------------------------

*Maximum flow on each HVDC link (3.2.1.1)
HVDCLinkMaximumFlow(TradeDate,TradePeriod,DCBranch) $ (BranchClosedStatus(DCBranch) and UseHVDCLinkLimits)..
HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch) =l= Sum[Season $ TradePeriodSeason(TradeDate,TradePeriod,Season),BranchCapacity(DCBranch,Season)];

*Definition of losses on the HVDC link (3.2.1.2)
HVDCLinkLossDefinition(TradeDate,TradePeriod,DCBranch)..
HVDCLINKLOSSES(TradeDate,TradePeriod,DCBranch) =e=
sum[(LossSegment,Season) $ [ValidLossSegment(DCBranch,LossSegment) and TradePeriodSeason(TradeDate,TradePeriod,Season)], HVDCBreakPointMWLoss(DCBranch,LossSegment,Season)*LAMBDA(TradeDate,TradePeriod,DCBranch,LossSegment)];

*Definition of MW flow on the HVDC link (3.2.1.3)
HVDCLinkFlowDefinition(TradeDate,TradePeriod,DCBranch) $ BranchClosedStatus(DCBranch)..
HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch) =e=
sum[(LossSegment,Season) $ [ValidLossSegment(DCBranch,LossSegment) and TradePeriodSeason(TradeDate,TradePeriod,Season)], HVDCBreakPointMWFlow(DCBranch,LossSegment,Season)*LAMBDA(TradeDate,TradePeriod,DCBranch,LossSegment)];

*Definition of weighting factor (3.2.1.4)
LambdaDefinition(TradeDate,TradePeriod,DCBranch) $ BranchClosedStatus(DCBranch)..
sum[LossSegment $ ValidLossSegment(DCBranch,LossSegment), LAMBDA(TradeDate,TradePeriod,DCBranch,LossSegment)] =e= 1;

*-----------------------------HVDC TRANSMISSION EQUATION CONSTRAINTS END-------------------------------------------------------------------------------------------------------





*-----------------------------AC TRANSMISSION EQUATION CONSTRAINTS-------------------------------------------------------------------------------------------------------------

*1st definition of the net injection at buses corresponding to AC nodes (3.3.1.1)
ACNodeNetInjectionDefinition1(TradeDate,TradePeriod,Node)..
ACNODENETINJECTION(TradeDate,TradePeriod,Node) =e= 0
+ sum[(ACBranch,FlowDirection) $ [BranchNodeFr(ACBranch,Node) and BranchClosedStatus(ACBranch) and (ord(FlowDirection)=1)], ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection)]
+ sum[(ACBranch,FlowDirection) $ [BranchNodeTo(ACBranch,Node) and BranchClosedStatus(ACBranch) and (ord(FlowDirection)=2)], ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection)]
- sum[(ACBranch,FlowDirection) $ [BranchNodeFr(ACBranch,Node) and BranchClosedStatus(ACBranch) and (ord(FlowDirection)=2)], ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection)]
- sum[(ACBranch,FlowDirection) $ [BranchNodeTo(ACBranch,Node) and BranchClosedStatus(ACBranch) and (ord(FlowDirection)=1)], ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection)]
;

*2nd definition of the net injection at buses corresponding to AC nodes (3.3.1.2)
ACNodeNetInjectionDefinition2(TradeDate,TradePeriod,Node)..
ACNODENETINJECTION(TradeDate,TradePeriod,Node) =e= 0
+ sum[OfferNode(Offer,Node), GENERATION(TradeDate,TradePeriod,Offer)]
- TradePeriodNodeDemand(TradeDate,TradePeriod,Node)
+ DEFICITBUSGENERATION(TradeDate,TradePeriod,Node) - SURPLUSBUSGENERATION(TradeDate,TradePeriod,Node)
- sum[DCBranch $ [BranchNodeFr(DCBranch,Node) and BranchClosedStatus(DCBranch)], HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch) + 0.5 * BranchFixedLoss(DCBranch)]
+ sum[DCBranch $ [BranchNodeTo(DCBranch,Node) and BranchClosedStatus(DCBranch)], HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch) - HVDCLINKLOSSES(TradeDate,TradePeriod,DCBranch) - 0.5 * BranchFixedLoss(DCBranch)]
- sum[(ACBranch,FlowDirection) $ [BranchNodeFr(ACBranch,Node) and BranchClosedStatus(ACBranch) and (ord(FlowDirection)=2)], ACBRANCHLOSSESDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection) + 0.5 * BranchFixedLoss(ACBranch)]
- sum[(ACBranch,FlowDirection) $ [BranchNodeTo(ACBranch,Node) and BranchClosedStatus(ACBranch) and (ord(FlowDirection)=1)], ACBRANCHLOSSESDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection) + 0.5 * BranchFixedLoss(ACBranch)]
;

*Maximum flow on the AC branch (3.3.1.3)
ACBranchMaximumFlow(TradeDate,TradePeriod,ACBranch,FlowDirection) $ UseACBranchLimits..
ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection) =l= SURPLUSBRANCHFLOW(TradeDate,TradePeriod,ACBranch) + Sum[Season  $ TradePeriodSeason(TradeDate,TradePeriod,Season), BranchCapacity(ACBranch,Season)]
;

*Relationship between directed and undirected branch flow variables (3.3.1.4)
ACBranchFlowDefinition(TradeDate,TradePeriod,ACBranch) $ BranchClosedStatus(ACBranch)..
ACBRANCHFLOW(TradeDate,TradePeriod,ACBranch) =e= 0
+ sum(FlowDirection $ (ord(FlowDirection) = 1), ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection))
- sum(FlowDirection $ (ord(FlowDirection) = 2), ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection))
;

*Equation that describes the linear load flow (3.3.1.5)
LinearLoadFlow(TradeDate,TradePeriod,ACBranch) $ BranchClosedStatus(ACBranch)..
ACBRANCHFLOW(TradeDate,TradePeriod,ACBranch) =e=
BranchSusceptance(ACBranch) * sum[(FromNode,ToNode) $ [BranchNodeFr(ACBranch,FromNode) and BranchNodeTo(ACBranch,ToNode)] , ACNODEANGLE(TradeDate,TradePeriod,FromNode) - ACNODEANGLE(TradeDate,TradePeriod,ToNode)]
;

*Limit on each AC branch flow block (3.3.1.6)
ACBranchBlockLimit(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection) $ BranchClosedStatus(ACBranch)..
ACBRANCHFLOWBLOCKDIRECTED(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection) =l= Sum[Season $ TradePeriodSeason(TradeDate,TradePeriod,Season), ACBranchLossMW(ACBranch,LossSegment,Season)]
;

*Composition of the directed branch flow from the block branch flow (3.3.1.7)
ACDirectedBranchFlowDefinition(TradeDate,TradePeriod,ACBranch,FlowDirection) $ BranchClosedStatus(ACBranch)..
ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection) =e=
sum[LossSegment $ ValidLossSegment(ACBranch,LossSegment), ACBRANCHFLOWBLOCKDIRECTED(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection)]
;

*Calculation of the losses in each loss segment (3.3.1.8)
ACBranchLossCalculation(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection) $ BranchClosedStatus(ACBranch)..
ACBRANCHLOSSESBLOCKDIRECTED(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection) =e=
ACBRANCHFLOWBLOCKDIRECTED(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection) * Sum[Season $ TradePeriodSeason(TradeDate,TradePeriod,Season) , ACBranchLossFactor(ACBranch,LossSegment,Season)]
;

*Composition of the directed branch losses from the block branch losses (3.3.1.9)
ACDirectedBranchLossDefinition(TradeDate,TradePeriod,ACBranch,FlowDirection) $ BranchClosedStatus(ACBranch)..
ACBRANCHLOSSESDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection) =e=
sum[LossSegment $ ValidLossSegment(ACBranch,LossSegment), ACBRANCHLOSSESBLOCKDIRECTED(TradeDate,TradePeriod,ACBranch,LossSegment,FlowDirection)]
;

*Reference Node Angle equal to zero (3.3.1.10)
ACReferenceNodeAngle(TradeDate,TradePeriod,RefNode)..
ACNODEANGLE(TradeDate,TradePeriod,RefNode) =e= 0
;
*-----------------------------AC TRANSMISSION EQUATION CONSTRAINTS END-----------------------------------------------------------------------------------------------------




*-----------------------------BRANCH SECURITY CONSTRAINTS------------------------------------------------------------------------------------------------------------------

*Branch security constraint with LE sense (3.5.1.5a)
BranchSecurityConstraintLE(TradeDate,TradePeriod,BranchGroupConstraint) $ (BranchGroupConstraintSense(BranchGroupConstraint) = -1)..
sum[ACBranch, BranchGroupConstraintFactor(BranchGroupConstraint,ACBranch) * ACBRANCHFLOW(TradeDate,TradePeriod,ACBranch)]
+ sum[DCBranch, BranchGroupConstraintFactor(BranchGroupConstraint,DCBranch) * HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch)]
- SURPLUSBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint) =l=
BranchGroupConstraintLimit(BranchGroupConstraint)
;

*Branch security constraint with GE sense (3.5.1.5b)
BranchSecurityConstraintGE(TradeDate,TradePeriod,BranchGroupConstraint) $ (BranchGroupConstraintSense(BranchGroupConstraint) = 1)..
sum[ACBranch, BranchGroupConstraintFactor(BranchGroupConstraint,ACBranch) * ACBRANCHFLOW(TradeDate,TradePeriod,ACBranch)]
+ sum[DCBranch, BranchGroupConstraintFactor(BranchGroupConstraint,DCBranch) * HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch)]
+ DEFICITBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint) =g=
BranchGroupConstraintLimit(BranchGroupConstraint)
;

*Branch security constraint with EQ sense (3.5.1.5c)
BranchSecurityConstraintEQ(TradeDate,TradePeriod,BranchGroupConstraint) $ (BranchGroupConstraintSense(BranchGroupConstraint) = 0)..
sum[ACBranch, BranchGroupConstraintFactor(BranchGroupConstraint,ACBranch) * ACBRANCHFLOW(TradeDate,TradePeriod,ACBranch)]
+ sum[DCBranch, BranchGroupConstraintFactor(BranchGroupConstraint,DCBranch) * HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch)]
+ DEFICITBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint) - SURPLUSBRANCHSECURITYCONSTRAINT(TradeDate,TradePeriod,BranchGroupConstraint) =e=
BranchGroupConstraintLimit(BranchGroupConstraint)
;
*-----------------------------BRANCH SECURITY CONSTRAINTS END---------------------------------------------------------------------------------------------------------------




*-----------------------------SOS1 and SOS2 CONSTRAINTS---------------------------------------------------------------------------------------------------------------------

*Definition of the integer HVDC link flow variable SOS1 (3.8.2a)
HVDCLinkFlowIntegerDefinition1(TradeDate,TradePeriod) $ [UseBranchFlowMIP(TradeDate,TradePeriod) and ResolveCircularBranchFlows]..
sum[FlowDirection, HVDCLINKFLOWDIRECTION_INTEGER(TradeDate,TradePeriod,FlowDirection)] =e=
sum[DCBranch $ BranchClosedStatus(DCBranch), HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch)]
;

*Definition of the integer HVDC link flow variable SOS1(3.8.2b)
HVDCLinkFlowIntegerDefinition2(TradeDate,TradePeriod,FlowDirection) $ (UseBranchFlowMIP(TradeDate,TradePeriod) and ResolveCircularBranchFlows)..
HVDCLINKFLOWDIRECTION_INTEGER(TradeDate,TradePeriod,FlowDirection) =e=
sum[DCDirection(DCBranch,FlowDirection) $ BranchClosedStatus(DCBranch), HVDCLINKFLOW(TradeDate,TradePeriod,DCBranch)]
;

*Definition of weighting factor when branch integer constraints are needed SOS2(3.8.3a)
LambdaIntegerDefinition1(TradeDate,TradePeriod,DCBranch) $ (BranchClosedStatus(DCBranch) and UseBranchFlowMIP(TradeDate,TradePeriod) and ResolveHVDCNonPhysicalLosses)..
sum[LossSegment $ ValidLossSegment(DCBranch,LossSegment), LAMBDAINTEGER(TradeDate,TradePeriod,DCBranch,LossSegment)] =e= 1
;

*Definition of weighting factor when branch integer constraints are needed SOS2(3.8.3b)
LambdaIntegerDefinition2(TradeDate,TradePeriod,DCBranch,LossSegment) $ (BranchClosedStatus(DCBranch) and ValidLossSegment(DCBranch,LossSegment) and UseBranchFlowMIP(TradeDate,TradePeriod) and ResolveHVDCNonPhysicalLosses)..
LAMBDAINTEGER(TradeDate,TradePeriod,DCBranch,LossSegment) =e= LAMBDA(TradeDate,TradePeriod,DCBranch,LossSegment)
;

*Integer constraint to enforce a flow direction on loss AC branches in the presence of circular branch flows or non-physical losses SOS1(3.8.1a)
ACDirectedBranchFlowIntegerDefinition1(TradeDate,TradePeriod,ACBranch) $ [BranchClosedStatus(ACBranch) and BranchNumLossTranches(ACBranch) and UseBranchFlowMIP(TradeDate,TradePeriod) and ResolveCircularBranchFlows]..
sum[FlowDirection, ACBRANCHFLOWDIRECTED_INTEGER(TradeDate,TradePeriod,ACBranch,FlowDirection)] =e= sum[FlowDirection, ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection)]
;

*Integer constraint to enforce a flow direction on loss AC branches in the presence of circular branch flows or non-physical losses SOS1(3.8.1b)
ACDirectedBranchFlowIntegerDefinition2(TradeDate,TradePeriod,ACBranch,FlowDirection) $ [BranchClosedStatus(ACBranch) and BranchNumLossTranches(ACBranch) and UseBranchFlowMIP(TradeDate,TradePeriod) and ResolveCircularBranchFlows]..
ACBRANCHFLOWDIRECTED_INTEGER(TradeDate,TradePeriod,ACBranch,FlowDirection) =e= ACBRANCHFLOWDIRECTED(TradeDate,TradePeriod,ACBranch,FlowDirection)
;
*-----------------------------SOS1 and SOS2 CONSTRAINTS END-----------------------------------------------------------------------------------------------------------------



*Model declarations

Model VSPD_HRC /
*Objective function
ObjectiveFunction
*Offer and purchase definitions
GenerationClearedMax
GenerationOfferDefintion
SchemeRatio
SchemeGenerationRatio
*Network
HVDCLinkMaximumFlow, HVDCLinkLossDefinition, HVDCLinkFlowDefinition, LambdaDefinition
ACNodeNetInjectionDefinition1, ACNodeNetInjectionDefinition2
ACBranchMaximumFlow, ACBranchFlowDefinition, LinearLoadFlow
ACBranchBlockLimit, ACDirectedBranchFlowDefinition, ACBranchLossCalculation, ACDirectedBranchLossDefinition
*Branch security constraints
BranchSecurityConstraintLE, BranchSecurityConstraintGE, BranchSecurityConstraintEQ
* ViolationCost
TotalViolationCostDefinition
/;

Model VSPD_BranchFlowMIP_HRC /
*Objective function
ObjectiveFunction
*Offer and purchase definitions
GenerationClearedMax
GenerationOfferDefintion
*Network
HVDCLinkMaximumFlow, HVDCLinkLossDefinition, HVDCLinkFlowDefinition, LambdaDefinition
ACNodeNetInjectionDefinition1, ACNodeNetInjectionDefinition2
ACBranchMaximumFlow, ACBranchFlowDefinition, LinearLoadFlow
ACBranchBlockLimit, ACDirectedBranchFlowDefinition, ACBranchLossCalculation, ACDirectedBranchLossDefinition
ACDirectedBranchFlowIntegerDefinition1, ACDirectedBranchFlowIntegerDefinition2, HVDCLinkFlowIntegerDefinition1, HVDCLinkFlowIntegerDefinition2
LambdaIntegerDefinition1, LambdaIntegerDefinition2
*Branch security constraints
BranchSecurityConstraintLE, BranchSecurityConstraintGE, BranchSecurityConstraintEQ
* ViolationCost
TotalViolationCostDefinition
/;
